// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

#[allow(unused_imports)]
use auto::*;

pub type DateDay = u8;
pub type DateYear = u16;
pub type Time = i32;
pub type TimeSpan = i64;
