// DO NOT EDIT THIS FILE. IT WAS AUTOMATICALLY GENERATED BY:
//
//   ucd-generate property-values ucd-13.0.0 --include gc,script,scx,age,gcb,wb,sb
//
// Unicode version: 13.0.0.
//
// ucd-generate 0.2.8 is available on crates.io.

pub const PROPERTY_VALUES: &'static [(
    &'static str,
    &'static [(&'static str, &'static str)],
)] = &[
    (
        "Age",
        &[
            ("1.1", "V1_1"),
            ("10.0", "V10_0"),
            ("11.0", "V11_0"),
            ("12.0", "V12_0"),
            ("12.1", "V12_1"),
            ("13.0", "V13_0"),
            ("2.0", "V2_0"),
            ("2.1", "V2_1"),
            ("3.0", "V3_0"),
            ("3.1", "V3_1"),
            ("3.2", "V3_2"),
            ("4.0", "V4_0"),
            ("4.1", "V4_1"),
            ("5.0", "V5_0"),
            ("5.1", "V5_1"),
            ("5.2", "V5_2"),
            ("6.0", "V6_0"),
            ("6.1", "V6_1"),
            ("6.2", "V6_2"),
            ("6.3", "V6_3"),
            ("7.0", "V7_0"),
            ("8.0", "V8_0"),
            ("9.0", "V9_0"),
            ("na", "Unassigned"),
            ("unassigned", "Unassigned"),
            ("v100", "V10_0"),
            ("v11", "V1_1"),
            ("v110", "V11_0"),
            ("v120", "V12_0"),
            ("v121", "V12_1"),
            ("v130", "V13_0"),
            ("v20", "V2_0"),
            ("v21", "V2_1"),
            ("v30", "V3_0"),
            ("v31", "V3_1"),
            ("v32", "V3_2"),
            ("v40", "V4_0"),
            ("v41", "V4_1"),
            ("v50", "V5_0"),
            ("v51", "V5_1"),
            ("v52", "V5_2"),
            ("v60", "V6_0"),
            ("v61", "V6_1"),
            ("v62", "V6_2"),
            ("v63", "V6_3"),
            ("v70", "V7_0"),
            ("v80", "V8_0"),
            ("v90", "V9_0"),
        ],
    ),
    (
        "General_Category",
        &[
            ("c", "Other"),
            ("casedletter", "Cased_Letter"),
            ("cc", "Control"),
            ("cf", "Format"),
            ("closepunctuation", "Close_Punctuation"),
            ("cn", "Unassigned"),
            ("cntrl", "Control"),
            ("co", "Private_Use"),
            ("combiningmark", "Mark"),
            ("connectorpunctuation", "Connector_Punctuation"),
            ("control", "Control"),
            ("cs", "Surrogate"),
            ("currencysymbol", "Currency_Symbol"),
            ("dashpunctuation", "Dash_Punctuation"),
            ("decimalnumber", "Decimal_Number"),
            ("digit", "Decimal_Number"),
            ("enclosingmark", "Enclosing_Mark"),
            ("finalpunctuation", "Final_Punctuation"),
            ("format", "Format"),
            ("initialpunctuation", "Initial_Punctuation"),
            ("l", "Letter"),
            ("lc", "Cased_Letter"),
            ("letter", "Letter"),
            ("letternumber", "Letter_Number"),
            ("lineseparator", "Line_Separator"),
            ("ll", "Lowercase_Letter"),
            ("lm", "Modifier_Letter"),
            ("lo", "Other_Letter"),
            ("lowercaseletter", "Lowercase_Letter"),
            ("lt", "Titlecase_Letter"),
            ("lu", "Uppercase_Letter"),
            ("m", "Mark"),
            ("mark", "Mark"),
            ("mathsymbol", "Math_Symbol"),
            ("mc", "Spacing_Mark"),
            ("me", "Enclosing_Mark"),
            ("mn", "Nonspacing_Mark"),
            ("modifierletter", "Modifier_Letter"),
            ("modifiersymbol", "Modifier_Symbol"),
            ("n", "Number"),
            ("nd", "Decimal_Number"),
            ("nl", "Letter_Number"),
            ("no", "Other_Number"),
            ("nonspacingmark", "Nonspacing_Mark"),
            ("number", "Number"),
            ("openpunctuation", "Open_Punctuation"),
            ("other", "Other"),
            ("otherletter", "Other_Letter"),
            ("othernumber", "Other_Number"),
            ("otherpunctuation", "Other_Punctuation"),
            ("othersymbol", "Other_Symbol"),
            ("p", "Punctuation"),
            ("paragraphseparator", "Paragraph_Separator"),
            ("pc", "Connector_Punctuation"),
            ("pd", "Dash_Punctuation"),
            ("pe", "Close_Punctuation"),
            ("pf", "Final_Punctuation"),
            ("pi", "Initial_Punctuation"),
            ("po", "Other_Punctuation"),
            ("privateuse", "Private_Use"),
            ("ps", "Open_Punctuation"),
            ("punct", "Punctuation"),
            ("punctuation", "Punctuation"),
            ("s", "Symbol"),
            ("sc", "Currency_Symbol"),
            ("separator", "Separator"),
            ("sk", "Modifier_Symbol"),
            ("sm", "Math_Symbol"),
            ("so", "Other_Symbol"),
            ("spaceseparator", "Space_Separator"),
            ("spacingmark", "Spacing_Mark"),
            ("surrogate", "Surrogate"),
            ("symbol", "Symbol"),
            ("titlecaseletter", "Titlecase_Letter"),
            ("unassigned", "Unassigned"),
            ("uppercaseletter", "Uppercase_Letter"),
            ("z", "Separator"),
            ("zl", "Line_Separator"),
            ("zp", "Paragraph_Separator"),
            ("zs", "Space_Separator"),
        ],
    ),
    (
        "Grapheme_Cluster_Break",
        &[
            ("cn", "Control"),
            ("control", "Control"),
            ("cr", "CR"),
            ("eb", "E_Base"),
            ("ebase", "E_Base"),
            ("ebasegaz", "E_Base_GAZ"),
            ("ebg", "E_Base_GAZ"),
            ("em", "E_Modifier"),
            ("emodifier", "E_Modifier"),
            ("ex", "Extend"),
            ("extend", "Extend"),
            ("gaz", "Glue_After_Zwj"),
            ("glueafterzwj", "Glue_After_Zwj"),
            ("l", "L"),
            ("lf", "LF"),
            ("lv", "LV"),
            ("lvt", "LVT"),
            ("other", "Other"),
            ("pp", "Prepend"),
            ("prepend", "Prepend"),
            ("regionalindicator", "Regional_Indicator"),
            ("ri", "Regional_Indicator"),
            ("sm", "SpacingMark"),
            ("spacingmark", "SpacingMark"),
            ("t", "T"),
            ("v", "V"),
            ("xx", "Other"),
            ("zwj", "ZWJ"),
        ],
    ),
    (
        "Script",
        &[
            ("adlam", "Adlam"),
            ("adlm", "Adlam"),
            ("aghb", "Caucasian_Albanian"),
            ("ahom", "Ahom"),
            ("anatolianhieroglyphs", "Anatolian_Hieroglyphs"),
            ("arab", "Arabic"),
            ("arabic", "Arabic"),
            ("armenian", "Armenian"),
            ("armi", "Imperial_Aramaic"),
            ("armn", "Armenian"),
            ("avestan", "Avestan"),
            ("avst", "Avestan"),
            ("bali", "Balinese"),
            ("balinese", "Balinese"),
            ("bamu", "Bamum"),
            ("bamum", "Bamum"),
            ("bass", "Bassa_Vah"),
            ("bassavah", "Bassa_Vah"),
            ("batak", "Batak"),
            ("batk", "Batak"),
            ("beng", "Bengali"),
            ("bengali", "Bengali"),
            ("bhaiksuki", "Bhaiksuki"),
            ("bhks", "Bhaiksuki"),
            ("bopo", "Bopomofo"),
            ("bopomofo", "Bopomofo"),
            ("brah", "Brahmi"),
            ("brahmi", "Brahmi"),
            ("brai", "Braille"),
            ("braille", "Braille"),
            ("bugi", "Buginese"),
            ("buginese", "Buginese"),
            ("buhd", "Buhid"),
            ("buhid", "Buhid"),
            ("cakm", "Chakma"),
            ("canadianaboriginal", "Canadian_Aboriginal"),
            ("cans", "Canadian_Aboriginal"),
            ("cari", "Carian"),
            ("carian", "Carian"),
            ("caucasianalbanian", "Caucasian_Albanian"),
            ("chakma", "Chakma"),
            ("cham", "Cham"),
            ("cher", "Cherokee"),
            ("cherokee", "Cherokee"),
            ("chorasmian", "Chorasmian"),
            ("chrs", "Chorasmian"),
            ("common", "Common"),
            ("copt", "Coptic"),
            ("coptic", "Coptic"),
            ("cprt", "Cypriot"),
            ("cuneiform", "Cuneiform"),
            ("cypriot", "Cypriot"),
            ("cyrillic", "Cyrillic"),
            ("cyrl", "Cyrillic"),
            ("deseret", "Deseret"),
            ("deva", "Devanagari"),
            ("devanagari", "Devanagari"),
            ("diak", "Dives_Akuru"),
            ("divesakuru", "Dives_Akuru"),
            ("dogr", "Dogra"),
            ("dogra", "Dogra"),
            ("dsrt", "Deseret"),
            ("dupl", "Duployan"),
            ("duployan", "Duployan"),
            ("egyp", "Egyptian_Hieroglyphs"),
            ("egyptianhieroglyphs", "Egyptian_Hieroglyphs"),
            ("elba", "Elbasan"),
            ("elbasan", "Elbasan"),
            ("elym", "Elymaic"),
            ("elymaic", "Elymaic"),
            ("ethi", "Ethiopic"),
            ("ethiopic", "Ethiopic"),
            ("geor", "Georgian"),
            ("georgian", "Georgian"),
            ("glag", "Glagolitic"),
            ("glagolitic", "Glagolitic"),
            ("gong", "Gunjala_Gondi"),
            ("gonm", "Masaram_Gondi"),
            ("goth", "Gothic"),
            ("gothic", "Gothic"),
            ("gran", "Grantha"),
            ("grantha", "Grantha"),
            ("greek", "Greek"),
            ("grek", "Greek"),
            ("gujarati", "Gujarati"),
            ("gujr", "Gujarati"),
            ("gunjalagondi", "Gunjala_Gondi"),
            ("gurmukhi", "Gurmukhi"),
            ("guru", "Gurmukhi"),
            ("han", "Han"),
            ("hang", "Hangul"),
            ("hangul", "Hangul"),
            ("hani", "Han"),
            ("hanifirohingya", "Hanifi_Rohingya"),
            ("hano", "Hanunoo"),
            ("hanunoo", "Hanunoo"),
            ("hatr", "Hatran"),
            ("hatran", "Hatran"),
            ("hebr", "Hebrew"),
            ("hebrew", "Hebrew"),
            ("hira", "Hiragana"),
            ("hiragana", "Hiragana"),
            ("hluw", "Anatolian_Hieroglyphs"),
            ("hmng", "Pahawh_Hmong"),
            ("hmnp", "Nyiakeng_Puachue_Hmong"),
            ("hrkt", "Katakana_Or_Hiragana"),
            ("hung", "Old_Hungarian"),
            ("imperialaramaic", "Imperial_Aramaic"),
            ("inherited", "Inherited"),
            ("inscriptionalpahlavi", "Inscriptional_Pahlavi"),
            ("inscriptionalparthian", "Inscriptional_Parthian"),
            ("ital", "Old_Italic"),
            ("java", "Javanese"),
            ("javanese", "Javanese"),
            ("kaithi", "Kaithi"),
            ("kali", "Kayah_Li"),
            ("kana", "Katakana"),
            ("kannada", "Kannada"),
            ("katakana", "Katakana"),
            ("katakanaorhiragana", "Katakana_Or_Hiragana"),
            ("kayahli", "Kayah_Li"),
            ("khar", "Kharoshthi"),
            ("kharoshthi", "Kharoshthi"),
            ("khitansmallscript", "Khitan_Small_Script"),
            ("khmer", "Khmer"),
            ("khmr", "Khmer"),
            ("khoj", "Khojki"),
            ("khojki", "Khojki"),
            ("khudawadi", "Khudawadi"),
            ("kits", "Khitan_Small_Script"),
            ("knda", "Kannada"),
            ("kthi", "Kaithi"),
            ("lana", "Tai_Tham"),
            ("lao", "Lao"),
            ("laoo", "Lao"),
            ("latin", "Latin"),
            ("latn", "Latin"),
            ("lepc", "Lepcha"),
            ("lepcha", "Lepcha"),
            ("limb", "Limbu"),
            ("limbu", "Limbu"),
            ("lina", "Linear_A"),
            ("linb", "Linear_B"),
            ("lineara", "Linear_A"),
            ("linearb", "Linear_B"),
            ("lisu", "Lisu"),
            ("lyci", "Lycian"),
            ("lycian", "Lycian"),
            ("lydi", "Lydian"),
            ("lydian", "Lydian"),
            ("mahajani", "Mahajani"),
            ("mahj", "Mahajani"),
            ("maka", "Makasar"),
            ("makasar", "Makasar"),
            ("malayalam", "Malayalam"),
            ("mand", "Mandaic"),
            ("mandaic", "Mandaic"),
            ("mani", "Manichaean"),
            ("manichaean", "Manichaean"),
            ("marc", "Marchen"),
            ("marchen", "Marchen"),
            ("masaramgondi", "Masaram_Gondi"),
            ("medefaidrin", "Medefaidrin"),
            ("medf", "Medefaidrin"),
            ("meeteimayek", "Meetei_Mayek"),
            ("mend", "Mende_Kikakui"),
            ("mendekikakui", "Mende_Kikakui"),
            ("merc", "Meroitic_Cursive"),
            ("mero", "Meroitic_Hieroglyphs"),
            ("meroiticcursive", "Meroitic_Cursive"),
            ("meroitichieroglyphs", "Meroitic_Hieroglyphs"),
            ("miao", "Miao"),
            ("mlym", "Malayalam"),
            ("modi", "Modi"),
            ("mong", "Mongolian"),
            ("mongolian", "Mongolian"),
            ("mro", "Mro"),
            ("mroo", "Mro"),
            ("mtei", "Meetei_Mayek"),
            ("mult", "Multani"),
            ("multani", "Multani"),
            ("myanmar", "Myanmar"),
            ("mymr", "Myanmar"),
            ("nabataean", "Nabataean"),
            ("nand", "Nandinagari"),
            ("nandinagari", "Nandinagari"),
            ("narb", "Old_North_Arabian"),
            ("nbat", "Nabataean"),
            ("newa", "Newa"),
            ("newtailue", "New_Tai_Lue"),
            ("nko", "Nko"),
            ("nkoo", "Nko"),
            ("nshu", "Nushu"),
            ("nushu", "Nushu"),
            ("nyiakengpuachuehmong", "Nyiakeng_Puachue_Hmong"),
            ("ogam", "Ogham"),
            ("ogham", "Ogham"),
            ("olchiki", "Ol_Chiki"),
            ("olck", "Ol_Chiki"),
            ("oldhungarian", "Old_Hungarian"),
            ("olditalic", "Old_Italic"),
            ("oldnortharabian", "Old_North_Arabian"),
            ("oldpermic", "Old_Permic"),
            ("oldpersian", "Old_Persian"),
            ("oldsogdian", "Old_Sogdian"),
            ("oldsoutharabian", "Old_South_Arabian"),
            ("oldturkic", "Old_Turkic"),
            ("oriya", "Oriya"),
            ("orkh", "Old_Turkic"),
            ("orya", "Oriya"),
            ("osage", "Osage"),
            ("osge", "Osage"),
            ("osma", "Osmanya"),
            ("osmanya", "Osmanya"),
            ("pahawhhmong", "Pahawh_Hmong"),
            ("palm", "Palmyrene"),
            ("palmyrene", "Palmyrene"),
            ("pauc", "Pau_Cin_Hau"),
            ("paucinhau", "Pau_Cin_Hau"),
            ("perm", "Old_Permic"),
            ("phag", "Phags_Pa"),
            ("phagspa", "Phags_Pa"),
            ("phli", "Inscriptional_Pahlavi"),
            ("phlp", "Psalter_Pahlavi"),
            ("phnx", "Phoenician"),
            ("phoenician", "Phoenician"),
            ("plrd", "Miao"),
            ("prti", "Inscriptional_Parthian"),
            ("psalterpahlavi", "Psalter_Pahlavi"),
            ("qaac", "Coptic"),
            ("qaai", "Inherited"),
            ("rejang", "Rejang"),
            ("rjng", "Rejang"),
            ("rohg", "Hanifi_Rohingya"),
            ("runic", "Runic"),
            ("runr", "Runic"),
            ("samaritan", "Samaritan"),
            ("samr", "Samaritan"),
            ("sarb", "Old_South_Arabian"),
            ("saur", "Saurashtra"),
            ("saurashtra", "Saurashtra"),
            ("sgnw", "SignWriting"),
            ("sharada", "Sharada"),
            ("shavian", "Shavian"),
            ("shaw", "Shavian"),
            ("shrd", "Sharada"),
            ("sidd", "Siddham"),
            ("siddham", "Siddham"),
            ("signwriting", "SignWriting"),
            ("sind", "Khudawadi"),
            ("sinh", "Sinhala"),
            ("sinhala", "Sinhala"),
            ("sogd", "Sogdian"),
            ("sogdian", "Sogdian"),
            ("sogo", "Old_Sogdian"),
            ("sora", "Sora_Sompeng"),
            ("sorasompeng", "Sora_Sompeng"),
            ("soyo", "Soyombo"),
            ("soyombo", "Soyombo"),
            ("sund", "Sundanese"),
            ("sundanese", "Sundanese"),
            ("sylo", "Syloti_Nagri"),
            ("sylotinagri", "Syloti_Nagri"),
            ("syrc", "Syriac"),
            ("syriac", "Syriac"),
            ("tagalog", "Tagalog"),
            ("tagb", "Tagbanwa"),
            ("tagbanwa", "Tagbanwa"),
            ("taile", "Tai_Le"),
            ("taitham", "Tai_Tham"),
            ("taiviet", "Tai_Viet"),
            ("takr", "Takri"),
            ("takri", "Takri"),
            ("tale", "Tai_Le"),
            ("talu", "New_Tai_Lue"),
            ("tamil", "Tamil"),
            ("taml", "Tamil"),
            ("tang", "Tangut"),
            ("tangut", "Tangut"),
            ("tavt", "Tai_Viet"),
            ("telu", "Telugu"),
            ("telugu", "Telugu"),
            ("tfng", "Tifinagh"),
            ("tglg", "Tagalog"),
            ("thaa", "Thaana"),
            ("thaana", "Thaana"),
            ("thai", "Thai"),
            ("tibetan", "Tibetan"),
            ("tibt", "Tibetan"),
            ("tifinagh", "Tifinagh"),
            ("tirh", "Tirhuta"),
            ("tirhuta", "Tirhuta"),
            ("ugar", "Ugaritic"),
            ("ugaritic", "Ugaritic"),
            ("unknown", "Unknown"),
            ("vai", "Vai"),
            ("vaii", "Vai"),
            ("wancho", "Wancho"),
            ("wara", "Warang_Citi"),
            ("warangciti", "Warang_Citi"),
            ("wcho", "Wancho"),
            ("xpeo", "Old_Persian"),
            ("xsux", "Cuneiform"),
            ("yezi", "Yezidi"),
            ("yezidi", "Yezidi"),
            ("yi", "Yi"),
            ("yiii", "Yi"),
            ("zanabazarsquare", "Zanabazar_Square"),
            ("zanb", "Zanabazar_Square"),
            ("zinh", "Inherited"),
            ("zyyy", "Common"),
            ("zzzz", "Unknown"),
        ],
    ),
    (
        "Script_Extensions",
        &[
            ("adlam", "Adlam"),
            ("adlm", "Adlam"),
            ("aghb", "Caucasian_Albanian"),
            ("ahom", "Ahom"),
            ("anatolianhieroglyphs", "Anatolian_Hieroglyphs"),
            ("arab", "Arabic"),
            ("arabic", "Arabic"),
            ("armenian", "Armenian"),
            ("armi", "Imperial_Aramaic"),
            ("armn", "Armenian"),
            ("avestan", "Avestan"),
            ("avst", "Avestan"),
            ("bali", "Balinese"),
            ("balinese", "Balinese"),
            ("bamu", "Bamum"),
            ("bamum", "Bamum"),
            ("bass", "Bassa_Vah"),
            ("bassavah", "Bassa_Vah"),
            ("batak", "Batak"),
            ("batk", "Batak"),
            ("beng", "Bengali"),
            ("bengali", "Bengali"),
            ("bhaiksuki", "Bhaiksuki"),
            ("bhks", "Bhaiksuki"),
            ("bopo", "Bopomofo"),
            ("bopomofo", "Bopomofo"),
            ("brah", "Brahmi"),
            ("brahmi", "Brahmi"),
            ("brai", "Braille"),
            ("braille", "Braille"),
            ("bugi", "Buginese"),
            ("buginese", "Buginese"),
            ("buhd", "Buhid"),
            ("buhid", "Buhid"),
            ("cakm", "Chakma"),
            ("canadianaboriginal", "Canadian_Aboriginal"),
            ("cans", "Canadian_Aboriginal"),
            ("cari", "Carian"),
            ("carian", "Carian"),
            ("caucasianalbanian", "Caucasian_Albanian"),
            ("chakma", "Chakma"),
            ("cham", "Cham"),
            ("cher", "Cherokee"),
            ("cherokee", "Cherokee"),
            ("chorasmian", "Chorasmian"),
            ("chrs", "Chorasmian"),
            ("common", "Common"),
            ("copt", "Coptic"),
            ("coptic", "Coptic"),
            ("cprt", "Cypriot"),
            ("cuneiform", "Cuneiform"),
            ("cypriot", "Cypriot"),
            ("cyrillic", "Cyrillic"),
            ("cyrl", "Cyrillic"),
            ("deseret", "Deseret"),
            ("deva", "Devanagari"),
            ("devanagari", "Devanagari"),
            ("diak", "Dives_Akuru"),
            ("divesakuru", "Dives_Akuru"),
            ("dogr", "Dogra"),
            ("dogra", "Dogra"),
            ("dsrt", "Deseret"),
            ("dupl", "Duployan"),
            ("duployan", "Duployan"),
            ("egyp", "Egyptian_Hieroglyphs"),
            ("egyptianhieroglyphs", "Egyptian_Hieroglyphs"),
            ("elba", "Elbasan"),
            ("elbasan", "Elbasan"),
            ("elym", "Elymaic"),
            ("elymaic", "Elymaic"),
            ("ethi", "Ethiopic"),
            ("ethiopic", "Ethiopic"),
            ("geor", "Georgian"),
            ("georgian", "Georgian"),
            ("glag", "Glagolitic"),
            ("glagolitic", "Glagolitic"),
            ("gong", "Gunjala_Gondi"),
            ("gonm", "Masaram_Gondi"),
            ("goth", "Gothic"),
            ("gothic", "Gothic"),
            ("gran", "Grantha"),
            ("grantha", "Grantha"),
            ("greek", "Greek"),
            ("grek", "Greek"),
            ("gujarati", "Gujarati"),
            ("gujr", "Gujarati"),
            ("gunjalagondi", "Gunjala_Gondi"),
            ("gurmukhi", "Gurmukhi"),
            ("guru", "Gurmukhi"),
            ("han", "Han"),
            ("hang", "Hangul"),
            ("hangul", "Hangul"),
            ("hani", "Han"),
            ("hanifirohingya", "Hanifi_Rohingya"),
            ("hano", "Hanunoo"),
            ("hanunoo", "Hanunoo"),
            ("hatr", "Hatran"),
            ("hatran", "Hatran"),
            ("hebr", "Hebrew"),
            ("hebrew", "Hebrew"),
            ("hira", "Hiragana"),
            ("hiragana", "Hiragana"),
            ("hluw", "Anatolian_Hieroglyphs"),
            ("hmng", "Pahawh_Hmong"),
            ("hmnp", "Nyiakeng_Puachue_Hmong"),
            ("hrkt", "Katakana_Or_Hiragana"),
            ("hung", "Old_Hungarian"),
            ("imperialaramaic", "Imperial_Aramaic"),
            ("inherited", "Inherited"),
            ("inscriptionalpahlavi", "Inscriptional_Pahlavi"),
            ("inscriptionalparthian", "Inscriptional_Parthian"),
            ("ital", "Old_Italic"),
            ("java", "Javanese"),
            ("javanese", "Javanese"),
            ("kaithi", "Kaithi"),
            ("kali", "Kayah_Li"),
            ("kana", "Katakana"),
            ("kannada", "Kannada"),
            ("katakana", "Katakana"),
            ("katakanaorhiragana", "Katakana_Or_Hiragana"),
            ("kayahli", "Kayah_Li"),
            ("khar", "Kharoshthi"),
            ("kharoshthi", "Kharoshthi"),
            ("khitansmallscript", "Khitan_Small_Script"),
            ("khmer", "Khmer"),
            ("khmr", "Khmer"),
            ("khoj", "Khojki"),
            ("khojki", "Khojki"),
            ("khudawadi", "Khudawadi"),
            ("kits", "Khitan_Small_Script"),
            ("knda", "Kannada"),
            ("kthi", "Kaithi"),
            ("lana", "Tai_Tham"),
            ("lao", "Lao"),
            ("laoo", "Lao"),
            ("latin", "Latin"),
            ("latn", "Latin"),
            ("lepc", "Lepcha"),
            ("lepcha", "Lepcha"),
            ("limb", "Limbu"),
            ("limbu", "Limbu"),
            ("lina", "Linear_A"),
            ("linb", "Linear_B"),
            ("lineara", "Linear_A"),
            ("linearb", "Linear_B"),
            ("lisu", "Lisu"),
            ("lyci", "Lycian"),
            ("lycian", "Lycian"),
            ("lydi", "Lydian"),
            ("lydian", "Lydian"),
            ("mahajani", "Mahajani"),
            ("mahj", "Mahajani"),
            ("maka", "Makasar"),
            ("makasar", "Makasar"),
            ("malayalam", "Malayalam"),
            ("mand", "Mandaic"),
            ("mandaic", "Mandaic"),
            ("mani", "Manichaean"),
            ("manichaean", "Manichaean"),
            ("marc", "Marchen"),
            ("marchen", "Marchen"),
            ("masaramgondi", "Masaram_Gondi"),
            ("medefaidrin", "Medefaidrin"),
            ("medf", "Medefaidrin"),
            ("meeteimayek", "Meetei_Mayek"),
            ("mend", "Mende_Kikakui"),
            ("mendekikakui", "Mende_Kikakui"),
            ("merc", "Meroitic_Cursive"),
            ("mero", "Meroitic_Hieroglyphs"),
            ("meroiticcursive", "Meroitic_Cursive"),
            ("meroitichieroglyphs", "Meroitic_Hieroglyphs"),
            ("miao", "Miao"),
            ("mlym", "Malayalam"),
            ("modi", "Modi"),
            ("mong", "Mongolian"),
            ("mongolian", "Mongolian"),
            ("mro", "Mro"),
            ("mroo", "Mro"),
            ("mtei", "Meetei_Mayek"),
            ("mult", "Multani"),
            ("multani", "Multani"),
            ("myanmar", "Myanmar"),
            ("mymr", "Myanmar"),
            ("nabataean", "Nabataean"),
            ("nand", "Nandinagari"),
            ("nandinagari", "Nandinagari"),
            ("narb", "Old_North_Arabian"),
            ("nbat", "Nabataean"),
            ("newa", "Newa"),
            ("newtailue", "New_Tai_Lue"),
            ("nko", "Nko"),
            ("nkoo", "Nko"),
            ("nshu", "Nushu"),
            ("nushu", "Nushu"),
            ("nyiakengpuachuehmong", "Nyiakeng_Puachue_Hmong"),
            ("ogam", "Ogham"),
            ("ogham", "Ogham"),
            ("olchiki", "Ol_Chiki"),
            ("olck", "Ol_Chiki"),
            ("oldhungarian", "Old_Hungarian"),
            ("olditalic", "Old_Italic"),
            ("oldnortharabian", "Old_North_Arabian"),
            ("oldpermic", "Old_Permic"),
            ("oldpersian", "Old_Persian"),
            ("oldsogdian", "Old_Sogdian"),
            ("oldsoutharabian", "Old_South_Arabian"),
            ("oldturkic", "Old_Turkic"),
            ("oriya", "Oriya"),
            ("orkh", "Old_Turkic"),
            ("orya", "Oriya"),
            ("osage", "Osage"),
            ("osge", "Osage"),
            ("osma", "Osmanya"),
            ("osmanya", "Osmanya"),
            ("pahawhhmong", "Pahawh_Hmong"),
            ("palm", "Palmyrene"),
            ("palmyrene", "Palmyrene"),
            ("pauc", "Pau_Cin_Hau"),
            ("paucinhau", "Pau_Cin_Hau"),
            ("perm", "Old_Permic"),
            ("phag", "Phags_Pa"),
            ("phagspa", "Phags_Pa"),
            ("phli", "Inscriptional_Pahlavi"),
            ("phlp", "Psalter_Pahlavi"),
            ("phnx", "Phoenician"),
            ("phoenician", "Phoenician"),
            ("plrd", "Miao"),
            ("prti", "Inscriptional_Parthian"),
            ("psalterpahlavi", "Psalter_Pahlavi"),
            ("qaac", "Coptic"),
            ("qaai", "Inherited"),
            ("rejang", "Rejang"),
            ("rjng", "Rejang"),
            ("rohg", "Hanifi_Rohingya"),
            ("runic", "Runic"),
            ("runr", "Runic"),
            ("samaritan", "Samaritan"),
            ("samr", "Samaritan"),
            ("sarb", "Old_South_Arabian"),
            ("saur", "Saurashtra"),
            ("saurashtra", "Saurashtra"),
            ("sgnw", "SignWriting"),
            ("sharada", "Sharada"),
            ("shavian", "Shavian"),
            ("shaw", "Shavian"),
            ("shrd", "Sharada"),
            ("sidd", "Siddham"),
            ("siddham", "Siddham"),
            ("signwriting", "SignWriting"),
            ("sind", "Khudawadi"),
            ("sinh", "Sinhala"),
            ("sinhala", "Sinhala"),
            ("sogd", "Sogdian"),
            ("sogdian", "Sogdian"),
            ("sogo", "Old_Sogdian"),
            ("sora", "Sora_Sompeng"),
            ("sorasompeng", "Sora_Sompeng"),
            ("soyo", "Soyombo"),
            ("soyombo", "Soyombo"),
            ("sund", "Sundanese"),
            ("sundanese", "Sundanese"),
            ("sylo", "Syloti_Nagri"),
            ("sylotinagri", "Syloti_Nagri"),
            ("syrc", "Syriac"),
            ("syriac", "Syriac"),
            ("tagalog", "Tagalog"),
            ("tagb", "Tagbanwa"),
            ("tagbanwa", "Tagbanwa"),
            ("taile", "Tai_Le"),
            ("taitham", "Tai_Tham"),
            ("taiviet", "Tai_Viet"),
            ("takr", "Takri"),
            ("takri", "Takri"),
            ("tale", "Tai_Le"),
            ("talu", "New_Tai_Lue"),
            ("tamil", "Tamil"),
            ("taml", "Tamil"),
            ("tang", "Tangut"),
            ("tangut", "Tangut"),
            ("tavt", "Tai_Viet"),
            ("telu", "Telugu"),
            ("telugu", "Telugu"),
            ("tfng", "Tifinagh"),
            ("tglg", "Tagalog"),
            ("thaa", "Thaana"),
            ("thaana", "Thaana"),
            ("thai", "Thai"),
            ("tibetan", "Tibetan"),
            ("tibt", "Tibetan"),
            ("tifinagh", "Tifinagh"),
            ("tirh", "Tirhuta"),
            ("tirhuta", "Tirhuta"),
            ("ugar", "Ugaritic"),
            ("ugaritic", "Ugaritic"),
            ("unknown", "Unknown"),
            ("vai", "Vai"),
            ("vaii", "Vai"),
            ("wancho", "Wancho"),
            ("wara", "Warang_Citi"),
            ("warangciti", "Warang_Citi"),
            ("wcho", "Wancho"),
            ("xpeo", "Old_Persian"),
            ("xsux", "Cuneiform"),
            ("yezi", "Yezidi"),
            ("yezidi", "Yezidi"),
            ("yi", "Yi"),
            ("yiii", "Yi"),
            ("zanabazarsquare", "Zanabazar_Square"),
            ("zanb", "Zanabazar_Square"),
            ("zinh", "Inherited"),
            ("zyyy", "Common"),
            ("zzzz", "Unknown"),
        ],
    ),
    (
        "Sentence_Break",
        &[
            ("at", "ATerm"),
            ("aterm", "ATerm"),
            ("cl", "Close"),
            ("close", "Close"),
            ("cr", "CR"),
            ("ex", "Extend"),
            ("extend", "Extend"),
            ("fo", "Format"),
            ("format", "Format"),
            ("le", "OLetter"),
            ("lf", "LF"),
            ("lo", "Lower"),
            ("lower", "Lower"),
            ("nu", "Numeric"),
            ("numeric", "Numeric"),
            ("oletter", "OLetter"),
            ("other", "Other"),
            ("sc", "SContinue"),
            ("scontinue", "SContinue"),
            ("se", "Sep"),
            ("sep", "Sep"),
            ("sp", "Sp"),
            ("st", "STerm"),
            ("sterm", "STerm"),
            ("up", "Upper"),
            ("upper", "Upper"),
            ("xx", "Other"),
        ],
    ),
    (
        "Word_Break",
        &[
            ("aletter", "ALetter"),
            ("cr", "CR"),
            ("doublequote", "Double_Quote"),
            ("dq", "Double_Quote"),
            ("eb", "E_Base"),
            ("ebase", "E_Base"),
            ("ebasegaz", "E_Base_GAZ"),
            ("ebg", "E_Base_GAZ"),
            ("em", "E_Modifier"),
            ("emodifier", "E_Modifier"),
            ("ex", "ExtendNumLet"),
            ("extend", "Extend"),
            ("extendnumlet", "ExtendNumLet"),
            ("fo", "Format"),
            ("format", "Format"),
            ("gaz", "Glue_After_Zwj"),
            ("glueafterzwj", "Glue_After_Zwj"),
            ("hebrewletter", "Hebrew_Letter"),
            ("hl", "Hebrew_Letter"),
            ("ka", "Katakana"),
            ("katakana", "Katakana"),
            ("le", "ALetter"),
            ("lf", "LF"),
            ("mb", "MidNumLet"),
            ("midletter", "MidLetter"),
            ("midnum", "MidNum"),
            ("midnumlet", "MidNumLet"),
            ("ml", "MidLetter"),
            ("mn", "MidNum"),
            ("newline", "Newline"),
            ("nl", "Newline"),
            ("nu", "Numeric"),
            ("numeric", "Numeric"),
            ("other", "Other"),
            ("regionalindicator", "Regional_Indicator"),
            ("ri", "Regional_Indicator"),
            ("singlequote", "Single_Quote"),
            ("sq", "Single_Quote"),
            ("wsegspace", "WSegSpace"),
            ("xx", "Other"),
            ("zwj", "ZWJ"),
        ],
    ),
];
