// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

use glib::translate::*;
use gst;
use gst_base;
use gst_video_sys;

glib_wrapper! {
    pub struct VideoFilter(Object<gst_video_sys::GstVideoFilter, gst_video_sys::GstVideoFilterClass, VideoFilterClass>) @extends gst_base::BaseTransform, gst::Element, gst::Object;

    match fn {
        get_type => || gst_video_sys::gst_video_filter_get_type(),
    }
}

impl VideoFilter {}

unsafe impl Send for VideoFilter {}
unsafe impl Sync for VideoFilter {}

pub const NONE_VIDEO_FILTER: Option<&VideoFilter> = None;
