// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

mod action;
pub use self::action::AtkActionExt;
pub use self::action::{Action, NONE_ACTION};

mod component;
pub use self::component::ComponentExt;
pub use self::component::{Component, NONE_COMPONENT};

mod document;
pub use self::document::DocumentExt;
pub use self::document::{Document, NONE_DOCUMENT};

mod editable_text;
pub use self::editable_text::EditableTextExt;
pub use self::editable_text::{EditableText, NONE_EDITABLE_TEXT};

mod gobject_accessible;
pub use self::gobject_accessible::GObjectAccessibleExt;
pub use self::gobject_accessible::{
    GObjectAccessible, GObjectAccessibleClass, NONE_GOBJECT_ACCESSIBLE,
};

mod hyperlink;
pub use self::hyperlink::HyperlinkExt;
pub use self::hyperlink::{Hyperlink, HyperlinkClass, NONE_HYPERLINK};

mod hyperlink_impl;
pub use self::hyperlink_impl::HyperlinkImplExt;
pub use self::hyperlink_impl::{HyperlinkImpl, NONE_HYPERLINK_IMPL};

mod hypertext;
pub use self::hypertext::HypertextExt;
pub use self::hypertext::{Hypertext, NONE_HYPERTEXT};

mod image;
pub use self::image::AtkImageExt;
pub use self::image::{Image, NONE_IMAGE};

mod misc;
pub use self::misc::AtkMiscExt;
pub use self::misc::{Misc, MiscClass, NONE_MISC};

mod no_op_object;
pub use self::no_op_object::{NoOpObject, NoOpObjectClass, NONE_NO_OP_OBJECT};

mod no_op_object_factory;
pub use self::no_op_object_factory::{
    NoOpObjectFactory, NoOpObjectFactoryClass, NONE_NO_OP_OBJECT_FACTORY,
};

mod object;
pub use self::object::AtkObjectExt;
pub use self::object::{Object, ObjectClass, NONE_OBJECT};

mod object_factory;
pub use self::object_factory::ObjectFactoryExt;
pub use self::object_factory::{ObjectFactory, ObjectFactoryClass, NONE_OBJECT_FACTORY};

mod plug;
pub use self::plug::AtkPlugExt;
pub use self::plug::{Plug, PlugClass, NONE_PLUG};

mod registry;
pub use self::registry::RegistryExt;
pub use self::registry::{Registry, RegistryClass, NONE_REGISTRY};

mod relation;
pub use self::relation::RelationExt;
pub use self::relation::{Relation, RelationClass, NONE_RELATION};

mod relation_set;
pub use self::relation_set::RelationSetExt;
pub use self::relation_set::{RelationSet, RelationSetClass, NONE_RELATION_SET};

mod selection;
pub use self::selection::SelectionExt;
pub use self::selection::{Selection, NONE_SELECTION};

mod socket;
pub use self::socket::AtkSocketExt;
pub use self::socket::{Socket, SocketClass, NONE_SOCKET};

mod state_set;
pub use self::state_set::StateSetExt;
pub use self::state_set::{StateSet, StateSetClass, NONE_STATE_SET};

mod streamable_content;
pub use self::streamable_content::StreamableContentExt;
pub use self::streamable_content::{StreamableContent, NONE_STREAMABLE_CONTENT};

mod table;
pub use self::table::TableExt;
pub use self::table::{Table, NONE_TABLE};

mod table_cell;
pub use self::table_cell::TableCellExt;
pub use self::table_cell::{TableCell, NONE_TABLE_CELL};

mod text;
pub use self::text::TextExt;
pub use self::text::{Text, NONE_TEXT};

mod util;
pub use self::util::{Util, UtilClass, NONE_UTIL};

mod value;
pub use self::value::ValueExt;
pub use self::value::{Value, NONE_VALUE};

mod window;
pub use self::window::AtkWindowExt;
pub use self::window::{Window, NONE_WINDOW};

mod range;
pub use self::range::Range;

mod rectangle;
pub use self::rectangle::Rectangle;

mod text_range;
pub use self::text_range::TextRange;

mod enums;
pub use self::enums::CoordType;
pub use self::enums::Layer;
pub use self::enums::RelationType;
pub use self::enums::Role;
#[cfg(any(feature = "v2_30", feature = "dox"))]
pub use self::enums::ScrollType;
pub use self::enums::StateType;
pub use self::enums::TextAttribute;
pub use self::enums::TextBoundary;
pub use self::enums::TextClipType;
pub use self::enums::TextGranularity;
pub use self::enums::ValueType;

mod flags;
pub use self::flags::HyperlinkStateFlags;

mod alias;
pub use self::alias::State;

#[doc(hidden)]
pub mod traits {
    pub use super::AtkActionExt;
    pub use super::AtkImageExt;
    pub use super::AtkMiscExt;
    pub use super::AtkObjectExt;
    pub use super::AtkPlugExt;
    pub use super::AtkSocketExt;
    pub use super::AtkWindowExt;
    pub use super::ComponentExt;
    pub use super::DocumentExt;
    pub use super::EditableTextExt;
    pub use super::GObjectAccessibleExt;
    pub use super::HyperlinkExt;
    pub use super::HyperlinkImplExt;
    pub use super::HypertextExt;
    pub use super::ObjectFactoryExt;
    pub use super::RegistryExt;
    pub use super::RelationExt;
    pub use super::RelationSetExt;
    pub use super::SelectionExt;
    pub use super::StateSetExt;
    pub use super::StreamableContentExt;
    pub use super::TableCellExt;
    pub use super::TableExt;
    pub use super::TextExt;
    pub use super::ValueExt;
}
