// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

#[allow(unused_imports)]
use auto::*;

pub type ClockTimeDiff = i64;
pub type ElementFactoryListType = u64;
