// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

use atk_sys;
use glib;
use glib::object::IsA;
use glib::translate::*;
use std::fmt;
use Object;

glib_wrapper! {
    pub struct ObjectFactory(Object<atk_sys::AtkObjectFactory, atk_sys::AtkObjectFactoryClass, ObjectFactoryClass>);

    match fn {
        get_type => || atk_sys::atk_object_factory_get_type(),
    }
}

pub const NONE_OBJECT_FACTORY: Option<&ObjectFactory> = None;

pub trait ObjectFactoryExt: 'static {
    fn create_accessible<P: IsA<glib::Object>>(&self, obj: &P) -> Option<Object>;

    fn get_accessible_type(&self) -> glib::types::Type;

    fn invalidate(&self);
}

impl<O: IsA<ObjectFactory>> ObjectFactoryExt for O {
    fn create_accessible<P: IsA<glib::Object>>(&self, obj: &P) -> Option<Object> {
        unsafe {
            from_glib_full(atk_sys::atk_object_factory_create_accessible(
                self.as_ref().to_glib_none().0,
                obj.as_ref().to_glib_none().0,
            ))
        }
    }

    fn get_accessible_type(&self) -> glib::types::Type {
        unsafe {
            from_glib(atk_sys::atk_object_factory_get_accessible_type(
                self.as_ref().to_glib_none().0,
            ))
        }
    }

    fn invalidate(&self) {
        unsafe {
            atk_sys::atk_object_factory_invalidate(self.as_ref().to_glib_none().0);
        }
    }
}

impl fmt::Display for ObjectFactory {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        write!(f, "ObjectFactory")
    }
}
