// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

mod action_row;
pub use self::action_row::ActionRowBuilder;
pub use self::action_row::ActionRowExt;
pub use self::action_row::{ActionRow, ActionRowClass, NONE_ACTION_ROW};

mod application_window;
pub use self::application_window::{
    ApplicationWindow, ApplicationWindowClass, NONE_APPLICATION_WINDOW,
};

mod avatar;
pub use self::avatar::AvatarBuilder;
pub use self::avatar::AvatarExt;
pub use self::avatar::{Avatar, AvatarClass, NONE_AVATAR};

mod carousel;
pub use self::carousel::CarouselBuilder;
pub use self::carousel::CarouselExt;
pub use self::carousel::{Carousel, CarouselClass, NONE_CAROUSEL};

mod carousel_indicator_dots;
pub use self::carousel_indicator_dots::CarouselIndicatorDotsExt;
pub use self::carousel_indicator_dots::{
    CarouselIndicatorDots, CarouselIndicatorDotsClass, NONE_CAROUSEL_INDICATOR_DOTS,
};

mod carousel_indicator_lines;
pub use self::carousel_indicator_lines::CarouselIndicatorLinesExt;
pub use self::carousel_indicator_lines::{
    CarouselIndicatorLines, CarouselIndicatorLinesClass, NONE_CAROUSEL_INDICATOR_LINES,
};

mod clamp;
pub use self::clamp::ClampBuilder;
pub use self::clamp::ClampExt;
pub use self::clamp::{Clamp, ClampClass, NONE_CLAMP};

mod combo_row;
pub use self::combo_row::ComboRowBuilder;
pub use self::combo_row::ComboRowExt;
pub use self::combo_row::{ComboRow, ComboRowClass, NONE_COMBO_ROW};

mod deck;
pub use self::deck::DeckBuilder;
pub use self::deck::DeckExt;
pub use self::deck::{Deck, DeckClass, NONE_DECK};

mod expander_row;
pub use self::expander_row::ExpanderRowBuilder;
pub use self::expander_row::ExpanderRowExt;
pub use self::expander_row::{ExpanderRow, ExpanderRowClass, NONE_EXPANDER_ROW};

mod header_bar;
pub use self::header_bar::HeaderBarBuilder;
pub use self::header_bar::HeaderBarExt;
pub use self::header_bar::{HeaderBar, HeaderBarClass, NONE_HEADER_BAR};

mod header_group;
pub use self::header_group::HeaderGroupBuilder;
pub use self::header_group::HeaderGroupExt;
pub use self::header_group::{HeaderGroup, HeaderGroupClass, NONE_HEADER_GROUP};

mod header_group_child;
pub use self::header_group_child::HeaderGroupChildExt;
pub use self::header_group_child::{
    HeaderGroupChild, HeaderGroupChildClass, NONE_HEADER_GROUP_CHILD,
};

mod keypad;
pub use self::keypad::KeypadBuilder;
pub use self::keypad::KeypadExt;
pub use self::keypad::{Keypad, KeypadClass, NONE_KEYPAD};

mod leaflet;
pub use self::leaflet::LeafletBuilder;
pub use self::leaflet::LeafletExt;
pub use self::leaflet::{Leaflet, LeafletClass, NONE_LEAFLET};

mod preferences_group;
pub use self::preferences_group::PreferencesGroupBuilder;
pub use self::preferences_group::PreferencesGroupExt;
pub use self::preferences_group::{
    PreferencesGroup, PreferencesGroupClass, NONE_PREFERENCES_GROUP,
};

mod preferences_page;
pub use self::preferences_page::PreferencesPageBuilder;
pub use self::preferences_page::PreferencesPageExt;
pub use self::preferences_page::{PreferencesPage, PreferencesPageClass, NONE_PREFERENCES_PAGE};

mod preferences_row;
pub use self::preferences_row::PreferencesRowBuilder;
pub use self::preferences_row::PreferencesRowExt;
pub use self::preferences_row::{PreferencesRow, PreferencesRowClass, NONE_PREFERENCES_ROW};

mod preferences_window;
pub use self::preferences_window::PreferencesWindowBuilder;
pub use self::preferences_window::PreferencesWindowExt;
pub use self::preferences_window::{
    PreferencesWindow, PreferencesWindowClass, NONE_PREFERENCES_WINDOW,
};

mod search_bar;
pub use self::search_bar::SearchBarBuilder;
pub use self::search_bar::SearchBarExt;
pub use self::search_bar::{SearchBar, SearchBarClass, NONE_SEARCH_BAR};

mod squeezer;
pub use self::squeezer::SqueezerBuilder;
pub use self::squeezer::SqueezerExt;
pub use self::squeezer::{Squeezer, SqueezerClass, NONE_SQUEEZER};

mod swipe_group;
pub use self::swipe_group::SwipeGroupExt;
pub use self::swipe_group::{SwipeGroup, SwipeGroupClass, NONE_SWIPE_GROUP};

mod swipe_tracker;
pub use self::swipe_tracker::SwipeTrackerBuilder;
pub use self::swipe_tracker::SwipeTrackerExt;
pub use self::swipe_tracker::{SwipeTracker, SwipeTrackerClass, NONE_SWIPE_TRACKER};

mod swipeable;
pub use self::swipeable::SwipeableExt;
pub use self::swipeable::{Swipeable, NONE_SWIPEABLE};

mod title_bar;
pub use self::title_bar::TitleBarBuilder;
pub use self::title_bar::TitleBarExt;
pub use self::title_bar::{TitleBar, TitleBarClass, NONE_TITLE_BAR};

mod value_object;
pub use self::value_object::ValueObjectExt;
pub use self::value_object::{ValueObject, ValueObjectClass, NONE_VALUE_OBJECT};

mod view_switcher;
pub use self::view_switcher::ViewSwitcherBuilder;
pub use self::view_switcher::ViewSwitcherExt;
pub use self::view_switcher::{ViewSwitcher, ViewSwitcherClass, NONE_VIEW_SWITCHER};

mod view_switcher_bar;
pub use self::view_switcher_bar::ViewSwitcherBarBuilder;
pub use self::view_switcher_bar::ViewSwitcherBarExt;
pub use self::view_switcher_bar::{ViewSwitcherBar, ViewSwitcherBarClass, NONE_VIEW_SWITCHER_BAR};

mod view_switcher_title;
pub use self::view_switcher_title::ViewSwitcherTitleBuilder;
pub use self::view_switcher_title::ViewSwitcherTitleExt;
pub use self::view_switcher_title::{
    ViewSwitcherTitle, ViewSwitcherTitleClass, NONE_VIEW_SWITCHER_TITLE,
};

mod window;
pub use self::window::{Window, WindowClass, NONE_WINDOW};

mod window_handle;
pub use self::window_handle::{WindowHandle, WindowHandleClass, NONE_WINDOW_HANDLE};

mod enums;
pub use self::enums::CenteringPolicy;
pub use self::enums::DeckTransitionType;
pub use self::enums::HeaderGroupChildType;
pub use self::enums::LeafletTransitionType;
pub use self::enums::NavigationDirection;
pub use self::enums::SqueezerTransitionType;
pub use self::enums::ViewSwitcherPolicy;

pub mod functions;

#[doc(hidden)]
pub mod traits {
    pub use super::ActionRowExt;
    pub use super::AvatarExt;
    pub use super::CarouselExt;
    pub use super::CarouselIndicatorDotsExt;
    pub use super::CarouselIndicatorLinesExt;
    pub use super::ClampExt;
    pub use super::ComboRowExt;
    pub use super::DeckExt;
    pub use super::ExpanderRowExt;
    pub use super::HeaderBarExt;
    pub use super::HeaderGroupChildExt;
    pub use super::HeaderGroupExt;
    pub use super::KeypadExt;
    pub use super::LeafletExt;
    pub use super::PreferencesGroupExt;
    pub use super::PreferencesPageExt;
    pub use super::PreferencesRowExt;
    pub use super::PreferencesWindowExt;
    pub use super::SearchBarExt;
    pub use super::SqueezerExt;
    pub use super::SwipeGroupExt;
    pub use super::SwipeTrackerExt;
    pub use super::SwipeableExt;
    pub use super::TitleBarExt;
    pub use super::ValueObjectExt;
    pub use super::ViewSwitcherBarExt;
    pub use super::ViewSwitcherExt;
    pub use super::ViewSwitcherTitleExt;
}
