// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

use gtk_sys;
use std::ffi::CStr;

#[cfg(any(feature = "v3_20", feature = "dox"))]
pub static LEVEL_BAR_OFFSET_FULL: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_LEVEL_BAR_OFFSET_FULL)
            .to_str()
            .unwrap()
    });
pub static LEVEL_BAR_OFFSET_HIGH: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_LEVEL_BAR_OFFSET_HIGH)
            .to_str()
            .unwrap()
    });
pub static LEVEL_BAR_OFFSET_LOW: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_LEVEL_BAR_OFFSET_LOW)
            .to_str()
            .unwrap()
    });
pub static PAPER_NAME_A3: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PAPER_NAME_A3).to_str().unwrap()
    });
pub static PAPER_NAME_A4: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PAPER_NAME_A4).to_str().unwrap()
    });
pub static PAPER_NAME_A5: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PAPER_NAME_A5).to_str().unwrap()
    });
pub static PAPER_NAME_B5: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PAPER_NAME_B5).to_str().unwrap()
    });
pub static PAPER_NAME_EXECUTIVE: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PAPER_NAME_EXECUTIVE)
            .to_str()
            .unwrap()
    });
pub static PAPER_NAME_LEGAL: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PAPER_NAME_LEGAL)
            .to_str()
            .unwrap()
    });
pub static PAPER_NAME_LETTER: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PAPER_NAME_LETTER)
            .to_str()
            .unwrap()
    });
pub static PRINT_SETTINGS_COLLATE: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PRINT_SETTINGS_COLLATE)
            .to_str()
            .unwrap()
    });
pub static PRINT_SETTINGS_DEFAULT_SOURCE: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PRINT_SETTINGS_DEFAULT_SOURCE)
            .to_str()
            .unwrap()
    });
pub static PRINT_SETTINGS_DITHER: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PRINT_SETTINGS_DITHER)
            .to_str()
            .unwrap()
    });
pub static PRINT_SETTINGS_DUPLEX: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PRINT_SETTINGS_DUPLEX)
            .to_str()
            .unwrap()
    });
pub static PRINT_SETTINGS_FINISHINGS: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PRINT_SETTINGS_FINISHINGS)
            .to_str()
            .unwrap()
    });
pub static PRINT_SETTINGS_MEDIA_TYPE: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PRINT_SETTINGS_MEDIA_TYPE)
            .to_str()
            .unwrap()
    });
pub static PRINT_SETTINGS_NUMBER_UP: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PRINT_SETTINGS_NUMBER_UP)
            .to_str()
            .unwrap()
    });
pub static PRINT_SETTINGS_NUMBER_UP_LAYOUT: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PRINT_SETTINGS_NUMBER_UP_LAYOUT)
            .to_str()
            .unwrap()
    });
pub static PRINT_SETTINGS_N_COPIES: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PRINT_SETTINGS_N_COPIES)
            .to_str()
            .unwrap()
    });
pub static PRINT_SETTINGS_ORIENTATION: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PRINT_SETTINGS_ORIENTATION)
            .to_str()
            .unwrap()
    });
pub static PRINT_SETTINGS_OUTPUT_BASENAME: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PRINT_SETTINGS_OUTPUT_BASENAME)
            .to_str()
            .unwrap()
    });
pub static PRINT_SETTINGS_OUTPUT_BIN: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PRINT_SETTINGS_OUTPUT_BIN)
            .to_str()
            .unwrap()
    });
pub static PRINT_SETTINGS_OUTPUT_DIR: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PRINT_SETTINGS_OUTPUT_DIR)
            .to_str()
            .unwrap()
    });
pub static PRINT_SETTINGS_OUTPUT_FILE_FORMAT: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PRINT_SETTINGS_OUTPUT_FILE_FORMAT)
            .to_str()
            .unwrap()
    });
pub static PRINT_SETTINGS_OUTPUT_URI: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PRINT_SETTINGS_OUTPUT_URI)
            .to_str()
            .unwrap()
    });
pub static PRINT_SETTINGS_PAGE_RANGES: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PRINT_SETTINGS_PAGE_RANGES)
            .to_str()
            .unwrap()
    });
pub static PRINT_SETTINGS_PAGE_SET: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PRINT_SETTINGS_PAGE_SET)
            .to_str()
            .unwrap()
    });
pub static PRINT_SETTINGS_PAPER_FORMAT: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PRINT_SETTINGS_PAPER_FORMAT)
            .to_str()
            .unwrap()
    });
pub static PRINT_SETTINGS_PAPER_HEIGHT: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PRINT_SETTINGS_PAPER_HEIGHT)
            .to_str()
            .unwrap()
    });
pub static PRINT_SETTINGS_PAPER_WIDTH: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PRINT_SETTINGS_PAPER_WIDTH)
            .to_str()
            .unwrap()
    });
pub static PRINT_SETTINGS_PRINTER: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PRINT_SETTINGS_PRINTER)
            .to_str()
            .unwrap()
    });
pub static PRINT_SETTINGS_PRINTER_LPI: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PRINT_SETTINGS_PRINTER_LPI)
            .to_str()
            .unwrap()
    });
pub static PRINT_SETTINGS_PRINT_PAGES: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PRINT_SETTINGS_PRINT_PAGES)
            .to_str()
            .unwrap()
    });
pub static PRINT_SETTINGS_QUALITY: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PRINT_SETTINGS_QUALITY)
            .to_str()
            .unwrap()
    });
pub static PRINT_SETTINGS_RESOLUTION: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PRINT_SETTINGS_RESOLUTION)
            .to_str()
            .unwrap()
    });
pub static PRINT_SETTINGS_RESOLUTION_X: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PRINT_SETTINGS_RESOLUTION_X)
            .to_str()
            .unwrap()
    });
pub static PRINT_SETTINGS_RESOLUTION_Y: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PRINT_SETTINGS_RESOLUTION_Y)
            .to_str()
            .unwrap()
    });
pub static PRINT_SETTINGS_REVERSE: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PRINT_SETTINGS_REVERSE)
            .to_str()
            .unwrap()
    });
pub static PRINT_SETTINGS_SCALE: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PRINT_SETTINGS_SCALE)
            .to_str()
            .unwrap()
    });
pub static PRINT_SETTINGS_USE_COLOR: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PRINT_SETTINGS_USE_COLOR)
            .to_str()
            .unwrap()
    });
pub static PRINT_SETTINGS_WIN32_DRIVER_EXTRA: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PRINT_SETTINGS_WIN32_DRIVER_EXTRA)
            .to_str()
            .unwrap()
    });
pub static PRINT_SETTINGS_WIN32_DRIVER_VERSION: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_PRINT_SETTINGS_WIN32_DRIVER_VERSION)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_ACCELERATOR: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_ACCELERATOR)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_ARROW: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_ARROW)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_BACKGROUND: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_BACKGROUND)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_BOTTOM: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_BOTTOM)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_BUTTON: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_BUTTON)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_CALENDAR: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_CALENDAR)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_CELL: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_CELL)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_CHECK: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_CHECK)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_COMBOBOX_ENTRY: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_COMBOBOX_ENTRY)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_CONTEXT_MENU: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_CONTEXT_MENU)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_CSD: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_CSD)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_CURSOR_HANDLE: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_CURSOR_HANDLE)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_DEFAULT: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_DEFAULT)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_DESTRUCTIVE_ACTION: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_DESTRUCTIVE_ACTION)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_DIM_LABEL: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_DIM_LABEL)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_DND: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_DND)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_DOCK: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_DOCK)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_ENTRY: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_ENTRY)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_ERROR: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_ERROR)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_EXPANDER: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_EXPANDER)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_FLAT: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_FLAT)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_FRAME: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_FRAME)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_GRIP: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_GRIP)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_HEADER: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_HEADER)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_HIGHLIGHT: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_HIGHLIGHT)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_HORIZONTAL: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_HORIZONTAL)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_IMAGE: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_IMAGE)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_INFO: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_INFO)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_INLINE_TOOLBAR: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_INLINE_TOOLBAR)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_INSERTION_CURSOR: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_INSERTION_CURSOR)
            .to_str()
            .unwrap()
    });
#[cfg(any(feature = "v3_16", feature = "dox"))]
pub static STYLE_CLASS_LABEL: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_LABEL)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_LEFT: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_LEFT)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_LEVEL_BAR: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_LEVEL_BAR)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_LINKED: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_LINKED)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_LIST: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_LIST)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_LIST_ROW: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_LIST_ROW)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_MARK: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_MARK)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_MENU: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_MENU)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_MENUBAR: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_MENUBAR)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_MENUITEM: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_MENUITEM)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_MESSAGE_DIALOG: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_MESSAGE_DIALOG)
            .to_str()
            .unwrap()
    });
#[cfg(any(feature = "v3_16", feature = "dox"))]
pub static STYLE_CLASS_MONOSPACE: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_MONOSPACE)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_NEEDS_ATTENTION: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_NEEDS_ATTENTION)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_NOTEBOOK: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_NOTEBOOK)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_OSD: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_OSD)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_OVERSHOOT: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_OVERSHOOT)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_PANE_SEPARATOR: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_PANE_SEPARATOR)
            .to_str()
            .unwrap()
    });
#[cfg(any(feature = "v3_16", feature = "dox"))]
pub static STYLE_CLASS_PAPER: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_PAPER)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_POPOVER: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_POPOVER)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_POPUP: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_POPUP)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_PRIMARY_TOOLBAR: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_PRIMARY_TOOLBAR)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_PROGRESSBAR: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_PROGRESSBAR)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_PULSE: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_PULSE)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_QUESTION: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_QUESTION)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_RADIO: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_RADIO)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_RAISED: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_RAISED)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_READ_ONLY: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_READ_ONLY)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_RIGHT: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_RIGHT)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_RUBBERBAND: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_RUBBERBAND)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_SCALE: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_SCALE)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_SCALE_HAS_MARKS_ABOVE: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_SCALE_HAS_MARKS_ABOVE)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_SCALE_HAS_MARKS_BELOW: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_SCALE_HAS_MARKS_BELOW)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_SCROLLBAR: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_SCROLLBAR)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_SCROLLBARS_JUNCTION: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_SCROLLBARS_JUNCTION)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_SEPARATOR: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_SEPARATOR)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_SIDEBAR: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_SIDEBAR)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_SLIDER: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_SLIDER)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_SPINBUTTON: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_SPINBUTTON)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_SPINNER: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_SPINNER)
            .to_str()
            .unwrap()
    });
#[cfg(any(feature = "v3_16", feature = "dox"))]
pub static STYLE_CLASS_STATUSBAR: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_STATUSBAR)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_SUBTITLE: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_SUBTITLE)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_SUGGESTED_ACTION: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_SUGGESTED_ACTION)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_TITLE: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_TITLE)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_TITLEBAR: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_TITLEBAR)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_TOOLBAR: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_TOOLBAR)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_TOOLTIP: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_TOOLTIP)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_TOP: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_TOP)
            .to_str()
            .unwrap()
    });
#[cfg(any(feature = "v3_16", feature = "dox"))]
pub static STYLE_CLASS_TOUCH_SELECTION: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_TOUCH_SELECTION)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_TROUGH: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_TROUGH)
            .to_str()
            .unwrap()
    });
#[cfg(any(feature = "v3_16", feature = "dox"))]
pub static STYLE_CLASS_UNDERSHOOT: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_UNDERSHOOT)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_VERTICAL: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_VERTICAL)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_VIEW: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_VIEW)
            .to_str()
            .unwrap()
    });
pub static STYLE_CLASS_WARNING: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_WARNING)
            .to_str()
            .unwrap()
    });
#[cfg(any(feature = "v3_16", feature = "dox"))]
pub static STYLE_CLASS_WIDE: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_CLASS_WIDE)
            .to_str()
            .unwrap()
    });
pub static STYLE_PROPERTY_BACKGROUND_COLOR: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_PROPERTY_BACKGROUND_COLOR)
            .to_str()
            .unwrap()
    });
pub static STYLE_PROPERTY_BACKGROUND_IMAGE: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_PROPERTY_BACKGROUND_IMAGE)
            .to_str()
            .unwrap()
    });
pub static STYLE_PROPERTY_BORDER_COLOR: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_PROPERTY_BORDER_COLOR)
            .to_str()
            .unwrap()
    });
pub static STYLE_PROPERTY_BORDER_RADIUS: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_PROPERTY_BORDER_RADIUS)
            .to_str()
            .unwrap()
    });
pub static STYLE_PROPERTY_BORDER_STYLE: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_PROPERTY_BORDER_STYLE)
            .to_str()
            .unwrap()
    });
pub static STYLE_PROPERTY_BORDER_WIDTH: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_PROPERTY_BORDER_WIDTH)
            .to_str()
            .unwrap()
    });
pub static STYLE_PROPERTY_COLOR: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_PROPERTY_COLOR)
            .to_str()
            .unwrap()
    });
pub static STYLE_PROPERTY_FONT: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_PROPERTY_FONT)
            .to_str()
            .unwrap()
    });
pub static STYLE_PROPERTY_MARGIN: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_PROPERTY_MARGIN)
            .to_str()
            .unwrap()
    });
pub static STYLE_PROPERTY_PADDING: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_PROPERTY_PADDING)
            .to_str()
            .unwrap()
    });
#[cfg_attr(feature = "v3_20", deprecated)]
pub static STYLE_REGION_COLUMN: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_REGION_COLUMN)
            .to_str()
            .unwrap()
    });
#[cfg_attr(feature = "v3_20", deprecated)]
pub static STYLE_REGION_COLUMN_HEADER: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_REGION_COLUMN_HEADER)
            .to_str()
            .unwrap()
    });
#[cfg_attr(feature = "v3_20", deprecated)]
pub static STYLE_REGION_ROW: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_REGION_ROW)
            .to_str()
            .unwrap()
    });
#[cfg_attr(feature = "v3_20", deprecated)]
pub static STYLE_REGION_TAB: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(gtk_sys::GTK_STYLE_REGION_TAB)
            .to_str()
            .unwrap()
    });
