// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

use gio_sys;
use glib::translate::*;

glib_wrapper! {
    #[derive(Debug, PartialEq, Eq, PartialOrd, Ord, Hash)]
    pub struct DBusPropertyInfo(Shared<gio_sys::GDBusPropertyInfo>);

    match fn {
        ref => |ptr| gio_sys::g_dbus_property_info_ref(ptr),
        unref => |ptr| gio_sys::g_dbus_property_info_unref(ptr),
        get_type => || gio_sys::g_dbus_property_info_get_type(),
    }
}
