.large-title {
   font-weight: 300;
   font-size: 24pt;
   letter-spacing: 0.2rem;
}
.page-title {
   font-weight: 600;
   font-size: 20pt;
}
.page-body {
   font-weight: 400;
   font-size: 12pt;
}
.video {
  opacity: 0;
}
.video.playing {
  opacity: 1;
  transition-property: opacity;
  transition-duration: 250ms;
  transition-timing-function: ease-in-out;
}

.last-page {
  background-color: #4a86cf; /*GNOME blue*/
}

.last-page .page-title, .last-page .page-body {
  color: white;
}
