// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

pub fn version() -> &'static str {
if cfg!(feature = "v3_24") {
        "3.24"
    } else if cfg!(feature = "v3_22") {
        "3.22"
    } else if cfg!(feature = "v3_20") {
        "3.20"
    } else if cfg!(feature = "v3_18") {
        "3.18"
    } else if cfg!(feature = "v3_16") {
        "3.16"
    } else {
        "3.14"
    }
}
