// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

use glib::translate::*;
use glib::GString;
use gst_player_sys;
use PlayerStreamInfo;

glib_wrapper! {
    pub struct PlayerSubtitleInfo(Object<gst_player_sys::GstPlayerSubtitleInfo, gst_player_sys::GstPlayerSubtitleInfoClass, PlayerSubtitleInfoClass>) @extends PlayerStreamInfo;

    match fn {
        get_type => || gst_player_sys::gst_player_subtitle_info_get_type(),
    }
}

impl PlayerSubtitleInfo {
    pub fn get_language(&self) -> Option<GString> {
        unsafe {
            from_glib_none(gst_player_sys::gst_player_subtitle_info_get_language(
                self.to_glib_none().0,
            ))
        }
    }
}

unsafe impl Send for PlayerSubtitleInfo {}
unsafe impl Sync for PlayerSubtitleInfo {}
