// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

use atk_sys;
use glib::object::Cast;
use glib::translate::*;
use std::fmt;
use ObjectFactory;

glib_wrapper! {
    pub struct NoOpObjectFactory(Object<atk_sys::AtkNoOpObjectFactory, atk_sys::AtkNoOpObjectFactoryClass, NoOpObjectFactoryClass>) @extends ObjectFactory;

    match fn {
        get_type => || atk_sys::atk_no_op_object_factory_get_type(),
    }
}

impl NoOpObjectFactory {
    pub fn new() -> NoOpObjectFactory {
        assert_initialized_main_thread!();
        unsafe {
            ObjectFactory::from_glib_full(atk_sys::atk_no_op_object_factory_new()).unsafe_cast()
        }
    }
}

impl Default for NoOpObjectFactory {
    fn default() -> Self {
        Self::new()
    }
}

pub const NONE_NO_OP_OBJECT_FACTORY: Option<&NoOpObjectFactory> = None;

impl fmt::Display for NoOpObjectFactory {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        write!(f, "NoOpObjectFactory")
    }
}
