// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

use glib::translate::*;
use gst;
use gst_base_sys;
use BaseSrc;

glib_wrapper! {
    pub struct PushSrc(Object<gst_base_sys::GstPushSrc, gst_base_sys::GstPushSrcClass, PushSrcClass>) @extends BaseSrc, gst::Element, gst::Object;

    match fn {
        get_type => || gst_base_sys::gst_push_src_get_type(),
    }
}

impl PushSrc {}

unsafe impl Send for PushSrc {}
unsafe impl Sync for PushSrc {}

pub const NONE_PUSH_SRC: Option<&PushSrc> = None;
