// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

use glib::object::Cast;
use glib::object::IsA;
use glib::signal::connect_raw;
use glib::signal::SignalHandlerId;
use glib::translate::*;
use glib::StaticType;
use glib::Value;
use glib_sys;
use gobject_sys;
use gst;
use gst_base;
use gst_video_sys;
use std::boxed::Box as Box_;
use std::mem::transmute;

glib_wrapper! {
    pub struct VideoSink(Object<gst_video_sys::GstVideoSink, gst_video_sys::GstVideoSinkClass, VideoSinkClass>) @extends gst_base::BaseSink, gst::Element, gst::Object;

    match fn {
        get_type => || gst_video_sys::gst_video_sink_get_type(),
    }
}

unsafe impl Send for VideoSink {}
unsafe impl Sync for VideoSink {}

pub const NONE_VIDEO_SINK: Option<&VideoSink> = None;

pub trait VideoSinkExt: 'static {
    fn get_property_show_preroll_frame(&self) -> bool;

    fn set_property_show_preroll_frame(&self, show_preroll_frame: bool);

    fn connect_property_show_preroll_frame_notify<F: Fn(&Self) + Send + Sync + 'static>(
        &self,
        f: F,
    ) -> SignalHandlerId;
}

impl<O: IsA<VideoSink>> VideoSinkExt for O {
    fn get_property_show_preroll_frame(&self) -> bool {
        unsafe {
            let mut value = Value::from_type(<bool as StaticType>::static_type());
            gobject_sys::g_object_get_property(
                self.to_glib_none().0 as *mut gobject_sys::GObject,
                b"show-preroll-frame\0".as_ptr() as *const _,
                value.to_glib_none_mut().0,
            );
            value
                .get()
                .expect("Return Value for property `show-preroll-frame` getter")
                .unwrap()
        }
    }

    fn set_property_show_preroll_frame(&self, show_preroll_frame: bool) {
        unsafe {
            gobject_sys::g_object_set_property(
                self.to_glib_none().0 as *mut gobject_sys::GObject,
                b"show-preroll-frame\0".as_ptr() as *const _,
                Value::from(&show_preroll_frame).to_glib_none().0,
            );
        }
    }

    fn connect_property_show_preroll_frame_notify<F: Fn(&Self) + Send + Sync + 'static>(
        &self,
        f: F,
    ) -> SignalHandlerId {
        unsafe extern "C" fn notify_show_preroll_frame_trampoline<
            P,
            F: Fn(&P) + Send + Sync + 'static,
        >(
            this: *mut gst_video_sys::GstVideoSink,
            _param_spec: glib_sys::gpointer,
            f: glib_sys::gpointer,
        ) where
            P: IsA<VideoSink>,
        {
            let f: &F = &*(f as *const F);
            f(&VideoSink::from_glib_borrow(this).unsafe_cast_ref())
        }
        unsafe {
            let f: Box_<F> = Box_::new(f);
            connect_raw(
                self.as_ptr() as *mut _,
                b"notify::show-preroll-frame\0".as_ptr() as *const _,
                Some(transmute::<_, unsafe extern "C" fn()>(
                    notify_show_preroll_frame_trampoline::<Self, F> as *const (),
                )),
                Box_::into_raw(f),
            )
        }
    }
}
