// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

mod binding;
pub use self::binding::{Binding, BindingClass};

mod flags;
pub use self::flags::BindingFlags;
pub use self::flags::ParamFlags;
pub use self::flags::SignalFlags;

#[doc(hidden)]
pub mod traits {}
